#include "MenuInput.h"

#include <Source/Gui/MainWnd.h>
#include <Source/Gui/Oscilloscope/Core/CoreOscilloscope.h>

CWndMenuInput::CWndMenuInput()
{
}

/*virtual*/ void CWndMenuInput::Create(CWnd *pParent, ui16 dwFlags) 
{
	CWnd::Create("CWndMenuInput", dwFlags, CRect(320-CWndMenuItem::MarginLeft, 20, 400, 240), pParent);
	m_itmCH1.Create(&Settings.CH1, this);
	m_itmCH2.Create(&Settings.CH2, this);
	m_itmCH3.Create(&Settings.CH3, this);
	m_itmCH4.Create(&Settings.CH4, this);
	m_itmTime.Create(&Settings.Time, this);
	m_itmWindow.Create(this);
	m_itmTrig.Create(this);
}

LINKERSECTION(".extra")
/*virtual*/ void CWndMenuInput::OnMessage(CWnd* pSender, ui16 code, ui32 data)
{
	// LAYOUT ENABLE/DISABLE FROM TOP MENU BAR
	if (code == ToWord('L', 'D') )
	{
		MainWnd.m_wndGraph.ShowWindow( SwHide );
		MainWnd.m_wndZoomBar.ShowWindow( SwHide );
		MainWnd.m_wndLReferences.ShowWindow( SwHide );
		MainWnd.m_wndTReferences.ShowWindow( SwHide );
	}

	if (code == ToWord('L', 'E') )
	{
		MainWnd.m_wndGraph.ShowWindow( SwShow );
		MainWnd.m_wndZoomBar.ShowWindow( SwShow );
		MainWnd.m_wndLReferences.ShowWindow( SwShow );
		MainWnd.m_wndTReferences.ShowWindow( SwShow );
	}


	// Trigger
	if (code == ToWord('m', 'r') )
	{
		m_wndListTrigger.Create( this );
		m_wndListTrigger.StartModal(&m_wndListTrigger.m_itmLevel);
	}
	if ( code == ToWord('o', 'k') && pSender->m_pParent == &m_wndListTrigger )
	{
		// done
		m_wndListTrigger.StopModal();
	}
	if ( code == ToWord('e', 'x') && pSender == &m_wndListTrigger )
	{
		// cancel
		m_wndListTrigger.StopModal();
	}
	if ( code == ToWord('u', 'p') && pSender->m_pParent == &m_wndListTrigger )
	{
		if ( pSender == &m_wndListTrigger.m_itmLevel )
			MainWnd.m_wndLReferences.Invalidate();
//		if ( pSender == &m_wndListTrigger.m_itmTime )
//			MainWnd.m_wndTReferences.Invalidate();
		// first 150 samples in buffer are for storing pre-trigger
		// samples. When no trigger is set, these samples are invalid.
		int nInvalid = Settings.Trig.Sync == CSettings::Trigger::_None ? 150 : 30;
		Settings.Time.InvalidFirst = nInvalid;
		if ( Settings.Time.Shift < nInvalid )
			Settings.Time.Shift = nInvalid;
		MainWnd.m_wndZoomBar.Invalidate();

		if ( pSender == &m_wndListTrigger.m_itmSync )
		{
			BIOS::ADC::Enable( true );
			BIOS::ADC::Restart();
		}

		Settings.Trig.State = (Settings.Trig.Sync == CSettings::Trigger::_Single) ? 
			CSettings::Trigger::_Wait : CSettings::Trigger::_Run;;

		CCoreOscilloscope::ConfigureTrigger();
		Settings.Trig.nLastChange = BIOS::SYS::GetTick();
		// update
		CWnd::GetTopModal().m_pPrevFocus->Invalidate();
	}

	// window
	if ( code == ToWord('w', 'u') )
	{
		MainWnd.m_wndTReferences.Invalidate();
		MainWnd.m_wndZoomBar.Invalidate();
		// if the osccilloscope window is freezed, inform to redraw it
		if ( !BIOS::ADC::Enabled() )
		{
			MainWnd.WindowMessage( CWnd::WmBroadcast, ToWord('d', 'g') );
		}
	}

	// Timebase
	if (code == ToWord('m', 't') )	// more timebase
	{
		m_wndComboSelector.Create( GetFocus()->m_pszId, WsVisible | WsModal, 
			CRect( 30, 100, 370, 140), RGB565(ffffff), (CValueProvider*)(NATIVEPTR)data, this);

		m_wndComboSelector.StartModal();
	}

	// Digital
	if (code == ToWord('m', 'd') )	// more input digital
	{
		m_wndListDInput.Create( (CSettings::DigitalChannel*)(NATIVEPTR)data, this );
		m_wndListDInput.StartModal( &m_wndListDInput.m_itmPosition );
	}
	if ( code == ToWord('o', 'k') && pSender->m_pParent == &m_wndListDInput ) m_wndListDInput.StopModal(); // done "OK"
	if ( code == ToWord('e', 'x') && pSender == &m_wndListDInput ) m_wndListDInput.StopModal(); // Cancel "EX"
	
	// Analog
	if (code == ToWord('m', 'a') )	// More Analog "ma"
	{
		m_wndListAInput.Create( (CSettings::AnalogChannel*)(NATIVEPTR)data, this );
		m_wndListAInput.StartModal( &m_wndListAInput.m_itmResolution );
	}
	if ( code == ToWord('o', 'k') && pSender->m_pParent == &m_wndListAInput ) m_wndListAInput.StopModal(); // done "OK"
	if ( code == ToWord('e', 'x') && pSender == &m_wndListAInput ) m_wndListAInput.StopModal(); // Cancel "EX"
	if ( code == ToWord('u', 'p') && pSender->m_pParent == &m_wndListAInput ) // "up"
	{
		if ( pSender == &m_wndListAInput.m_itmRed || pSender == &m_wndListAInput.m_itmGrn || pSender == &m_wndListAInput.m_itmBlu ) m_wndListAInput.m_itmColour.Invalidate();
		if ( pSender == &m_wndListAInput.m_itmPosition ) MainWnd.m_wndLReferences.Invalidate();
		//if ( pSender == &m_wndListAInput.m_itmProbe ) m_wndListAInput.m_itmResolution.Invalidate();  // FIXME ==> must update the "Resolution" part, but this is not working
		// update
		CWnd::GetTopModal().m_pPrevFocus->Invalidate();
		CCoreOscilloscope::ConfigureAdc();
	}
	if ( code == ToWord('u', 'p') && pSender->m_pParent == &m_wndListDInput ) // "up"
	{
		if ( pSender == &m_wndListDInput.m_itmRed || pSender == &m_wndListDInput.m_itmGrn || pSender == &m_wndListDInput.m_itmBlu ) m_wndListDInput.m_itmColour.Invalidate();
		if ( pSender == &m_wndListDInput.m_itmPosition ) MainWnd.m_wndLReferences.Invalidate();
		// update
		CWnd::GetTopModal().m_pPrevFocus->Invalidate();
	}
	// Selector
	if ( code == ToWord('l', 'e') )	// provider selector combo box "le"
	{
		CValueProvider* pProvider = (CValueProvider*)(NATIVEPTR)data;
		m_wndComboSelector.Create( GetFocus()->m_pszId, WsVisible | WsModal,CRect( 30, 100, 370, 140), RGB565(ffffff), pProvider, this);
		m_wndComboSelector.StartModal();
	}
	if ( code == ToWord('o', 'k') && pSender == &m_wndComboSelector ) m_wndComboSelector.StopModal();	// done "OK"
	if ( code == ToWord('e', 'x') && pSender == &m_wndComboSelector ) m_wndComboSelector.StopModal();	// cancel "EX"
	if ( code == ToWord('i', 'u') ) CCoreOscilloscope::ConfigureAdc();// update hardware gain according to new selection (menu's are updated anyway)
	if ( code == ToWord('r', 'u') )
	{
		Settings.Trig.nLastChange = BIOS::SYS::GetTick();
		MainWnd.m_wndLReferences.Invalidate();
		CCoreOscilloscope::ConfigureTrigger();
	}
}


